<?php

class Followthis_API_Wrapper {

	/**
	 * The API endpoint
	 * @var apiUrl
	 */
	var $apiURL = 'https://www.followthis.de/api/';

	/**
	 * Version of the API
	 * @var apiVersion
	 */
	var $apiVersion = '1.0';

	/**
	 * @since 1.0
	 */
	function __construct() {
	}


	/**
	 * builds the complete url with version and action
	 *
	 * @since 1.0
	 * @param string $action
	 */
	function createUrl( $action = '' ) {
		return $this->apiURL . $this->apiVersion . '/' . $action;
	}


	/**
	 * @param $name
	 * @param $url_prefix
	 * @since 1.0
	 * @return bool|object
	 */
	function registerPublisher( $name, $url_prefix ) {
		$url = $this->createUrl( 'publisher' );

		$data = array(
			'body'    => json_encode(
				array(
					'name'       => $name,
					'url_prefix' => $url_prefix
				)
			),
			'headers' => array(
				'Content-Type' => 'application/json'
			)
		);

		$request = wp_remote_post( $url, $data );

		if ( is_wp_error( $request ) )
			return false;

		$response = json_decode( wp_remote_retrieve_body( $request ) );

		if ( is_object( $response ) ) {
			return $response;
		} else {
			// unexpected response
			return false;
		}
	}

	/**
	 * @param $key
	 * @param $verify_url
	 *
	 * @since 1.0
	 * @return bool|object
	 */
	function publisherVerification( $key, $verify_url ) {
		$url = $this->createUrl( 'publisherverification' );

		$data = array(
			'body'    => json_encode(
				array(
					'key' => $key,
					'url' => $verify_url
				)
			),
			'headers' => array(
				'Content-Type' => 'application/json'
			)
		);

		$request = wp_remote_post( $url, $data );

		if ( is_wp_error( $request ) )
			return false;

		$response = json_decode( wp_remote_retrieve_body( $request ) );

		if ( is_object( $response ) ) {
			return $response;
		} else {
			// unexpected response
			return false;
		}
	}
}
