<?php
/**
 * This file holds all the meta boxes and the function to create the meta boxes for the wpLike2Get settings
 * page in the admin.
 *
 * @package followthis
 */

class Followthis_Meta_Box {

	var $settings_page = 'settings_page_followthis';

	/**
	 * @since 1.0
	 */
	function __construct() {
		/* Add the meta boxes for the settings page on the 'add_meta_boxes' hook. */
		add_action( 'add_meta_boxes', array( &$this, 'create_meta_boxes' ) );
	}


	/**
	 * Adds all the meta boxes to the followthis settings page in the WP admin.
	 *
	 * @since 1.0
	 */
	function create_meta_boxes() {
		/* Add the 'About' meta box. */
		add_meta_box( 'followthis-about', __( 'About', 'followthis' ), array( &$this, 'meta_box_display_about' ), $this->settings_page, 'side', 'default' );

		/* Add the 'Support' meta box. */
		add_meta_box( 'followthis-support', __( 'Support', 'followthis' ), array( &$this, 'meta_box_display_support' ), $this->settings_page, 'side', 'low' );

		/* Add the 'Support' meta box. */
		add_meta_box( 'followthis-general', __( 'General', 'followthis' ), array( &$this, 'meta_box_display_general' ), $this->settings_page, 'normal', 'high' );
	}

	/**
	 * Displays the about plugin meta box.
	 *
	 * @since 1.0
	 */
	function meta_box_display_about( $object, $box ) {

		$plugin_data = get_plugin_data( dirname( __FILE__ ) . '/followthis.php' ); ?>

		<p>
			<strong><?php _e( 'Version:', 'followthis' ); ?></strong> <?php echo $plugin_data['Version']; ?>
		</p>
		<p>
			<strong><?php _e( 'Description:', 'followthis' ); ?></strong>
		</p>
		<p>
			<?php echo $plugin_data['Description']; ?>
		</p>
	<?php
	}

	/**
	 * Displays the support meta box.
	 *
	 * @since 1.0
	 */
	function meta_box_display_support( $object, $box ) {
		?>
		<p>
			<?php printf( __( 'Support for this plugin is provided via our support %sforum%s. Or contact our team directly %spublisher-support@followthis.de%s.', 'followthis' ), '<a href="http://followthisorg.uservoice.com" title="' . __( 'followthis support forum', 'followthis' ) . '">', '</a>', '<a href="mailto:publisher-support@followthis.de">', '</a>' ); ?>
		</p>

	<?php
	}

	/**
	 * Display the general meta box
	 *
	 * @since 1.0
	 */
	function meta_box_display_general() {
		?>

		<table class="form-table">
			<tbody>
			<tr>
				<th>
					<label for="keywords"><?php _e( 'Standard Keywords / Topics', 'followthis' ) ?></label>
				</th>
				<td>
					<input type="text" class="large-text" name="followthis_options[keywords]" id="keywords" value="<?php echo Followthis::get_setting( 'keywords', 'followthis_options' ) ?>"/>
					<br/><label class="description" for="keywords"><?php _e( 'If you don\'t add tags to your post manually, these topics are taken into the followthis topic box. Please use "," between the topics.', 'followthis' ) ?></label>
				</td>
			</tr>

			<?php if( Followthis::get_setting( 'key' ) ) : ?>
			<tr>
				<th>
					<label for="register"><?php _e( 'Change profile', 'followthis' ) ?></label>
				</th>
				<td>
					<?php printf( __( 'via the following link you can always %schange your publisher profile information%s on followthis', 'followthis' ), '<a href="http://www.followthis.de/publisher_edit/' . Followthis::get_setting( 'key' ) . '" target="_blank">', '</a>'  ); ?></a>
				</td>
			</tr>
			<?php endif; ?>
			</tbody>
		</table>
	<?php
	}
}

$ft_metabox = new Followthis_Meta_Box();