<?php
/*
Plugin Name: followthis topic box
Plugin URI: http://followthis.org
Description: followthis topic box helps you gain frequent reader. User follow your topics with only 1 klick + will be allerted frequently if new content available
Version: 1.0.2
Author: followthis
Author URI: http://followthis.org
Author Email: publisher-support@followthis.de
License: GPL v3

  Copyright 2013 FollowThis Media UG (info@followthis.de)

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License, version 2, as 
  published by the Free Software Foundation.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

require_once( dirname( __FILE__ ) . '/Followthis_API_Wrapper.php' );
require_once( dirname( __FILE__ ) . '/Followthis_Admin_Options.php' );
require_once( dirname( __FILE__ ) . '/ajax.php' );

class Followthis {

	static $settings;

	var $api;

	/*--------------------------------------------*
	 * Constructor
	 *--------------------------------------------*/

	/**
	 * Initializes the plugin by setting localization, filters, and administration functions.
	 *
	 * @since 1.0
	 */
	function __construct() {

		$this->api = new Followthis_API_Wrapper();

		// Load plugin text domain
		add_action( 'init', array( &$this, 'plugin_textdomain' ) );

		if ( is_admin() ) {
			// Register admin styles and scripts
			add_action( 'admin_print_styles', array( &$this, 'register_admin_styles' ) );
			add_action( 'admin_enqueue_scripts', array( &$this, 'register_admin_scripts' ) );
		}

		if ( ! is_admin() ) {
			// Register site styles and scripts
			add_action( 'wp_enqueue_scripts', array( &$this, 'register_plugin_styles' ) );
			add_action( 'wp_enqueue_scripts', array( &$this, 'register_plugin_scripts' ) );
		}

		// Register hooks that are fired when the plugin is activated, deactivated, and uninstalled, respectively.
		register_activation_hook( __FILE__, array( &$this, 'activate' ) );
		register_deactivation_hook( __FILE__, array( &$this, 'deactivate' ) );
		register_uninstall_hook( __FILE__, array( &$this, 'uninstall' ) );

		/*
		 * Custom functionality for the plugin.
		 */
		// show warning if account can't be verified.
		add_action( 'admin_footer', array( &$this, 'not_verified_warning' ) );
		add_action( 'admin_footer', array( &$this, 'not_active_warning' ) );

		if ( is_admin() ) {
			add_action( 'add_meta_boxes', array( &$this, 'add_tagging_meta_box' ) );
			add_action( 'save_post', array( &$this, 'save_postdata' ) );
		} else {
			add_action( 'wp_head', array( &$this, 'add_ft_keywords' ) );
			add_filter( 'the_content', array( &$this, 'followthis_add_post_content' ) );
		}

	} // end constructor

	/**
	 * Fired when the plugin is activated.
	 *
	 * @param    boolean $network_wide    True if WPMU superadmin uses "Network Activate" action, false if WPMU is disabled or plugin is activated on an individual blog
	 *
	 * @since 1.0
	 */
	public function activate( $network_wide ) {

		if ( $network_wide )
			return false;

		if ( $this->get_setting( 'verified' ) ) {
			return false;
		}

		$register = $this->registerPublisher();
		if ( ! $register )
			return;

		if ( is_writeable( dirname( __FILE__ ) ) ) {
			$written = $this->createVerificationFile( md5( $register->key ), $register->verify_content );
			$this->set_setting( 'key', $register->key );
			$this->set_setting( 'verify_content', $register->verify_content );

			if ( $written != FALSE ) {
				$verification = $this->publishVerification( $register->key, plugins_url( md5( $register->key ) . '.html', __FILE__ ) );

				// if verification is failed
				if ( ! $verification )
					$this->set_setting( 'verified', false );
				else
					$this->set_setting( 'verified', true );
			}
		}
	} // end activate


	/**
	 * Get a setting for the followthis plugin.
	 *
	 * @since  1.0
	 */
	public static function get_setting( $option = '', $option_name = 'followthis' ) {
		if ( ! $option )
			return false;

//		if ( ! isset( self::$settings ) || ! is_array( self::$settings ) )
		self::$settings = get_option( $option_name );

		return self::$settings[$option];
	}

	/**
	 * Update a setting for the followthis plugin.
	 *
	 * @since  1.0
	 */
	static function set_setting( $option = '', $value, $option_name = 'followthis' ) {
		if ( ! $option )
			return false;

		if ( ! isset( self::$settings ) || ! is_array( self::$settings ) )
			self::$settings = get_option( $option_name );

		self::$settings[$option] = $value;
		update_option( $option_name, self::$settings );
	}


	/**
	 * Fired when the plugin is deactivated.
	 *
	 * @param    boolean $network_wide    True if WPMU superadmin uses "Network Activate" action, false if WPMU is disabled or plugin is activated on an individual blog
	 *
	 * @since 1.0
	 */
	public function deactivate( $network_wide ) {
	} // end deactivate

	/**
	 * Fired when the plugin is uninstalled.
	 *
	 * @param    boolean $network_wide    True if WPMU superadmin uses "Network Activate" action, false if WPMU is disabled or plugin is activated on an individual blog
	 *
	 * @since 1.0
	 */
	public function uninstall( $network_wide ) {
	} // end uninstall

	/**
	 * Loads the plugin text domain for translation
	 * @since 1.0
	 */
	public function plugin_textdomain() {

		$domain = 'followthis';
//		$locale = apply_filters( 'plugin_locale', get_locale(), $domain );
		load_plugin_textdomain( $domain, FALSE, dirname( plugin_basename( __FILE__ ) ) . '/languages' );

	} // end plugin_textdomain

	/**
	 * Registers and enqueues admin-specific styles.
	 * @since 1.0
	 */
	public function register_admin_styles() {

		wp_enqueue_style( 'followthis-admin-styles', plugins_url( 'followthis-topic-box/css/admin.css' ) );

	} // end register_admin_styles

	/**
	 * Registers and enqueues admin-specific JavaScript.
	 * @since 1.0
	 */
	public function register_admin_scripts() {

		wp_enqueue_script( 'followthis-admin-script', plugins_url( 'followthis-topic-box/js/admin.js' ) );

	} // end register_admin_scripts

	/**
	 * Registers and enqueues plugin-specific styles.
	 * @since 1.0
	 */
	public function register_plugin_styles() {
	} // end register_plugin_styles

	/**
	 * Registers and enqueues plugin-specific scripts.
	 * @since 1.0
	 */
	public function register_plugin_scripts() {
		if ( is_single() )
			wp_enqueue_script( 'followthis-plugin-script', plugins_url( 'followthis-topic-box/js/display.js' ), array( 'jquery' ) );

	} // end register_plugin_scripts

	/*--------------------------------------------*
	 * Core Functions
	 *---------------------------------------------*/

	/**
	 * Display an error message when publishers blog can't be verified
	 * @since 1.0
	 */
	function not_verified_warning() {
		if ( function_exists( 'is_network_admin' ) && is_network_admin() )
			return;

		if ( self::get_setting( 'verified' ) == 1 || self::get_setting( 'ignore_not_verified_warning' ) == 'ignore' ) {

		} else {
			echo '<div id="message" class="not_verified_warning error">';
			echo '<p><strong>' . __( 'Your followthis account can\'t be verified.', 'followthis' ) . '</strong> ' . sprintf( __( 'You may have done already an installation in the past. Please contact the %sfollowthis team%s.', 'followthis' ), '<a href="mailto:publisher-support@followthis.de">', '</a>' )
				. ' <a href="javascript:followthis_set_ignore(\'not_verified_warning\', \'not_verified_warning\', \'' . wp_create_nonce( 'followthis-ignore' ) . '\');" class="button">' . __( 'close', 'followthis' ) . '</a>'
				. '</p></div>';
		}
	}

	/**
	 * Display an error message when publisher is not active (not registered)
	 * @since 1.0
	 */
	function not_active_warning() {
		if ( function_exists( 'is_network_admin' ) && is_network_admin() )
			return;

		if ( ! $this->get_setting( 'verified' ) || $this->get_setting( 'ignore_not_active_warning' ) == 'ignore' )
			return;

		echo '<div id="message" class="not_active_warning error">';
		echo '<p><strong>' . __( 'Important:', 'followthis' ) . '</strong> ' . sprintf( __( 'Please check your followthis publisher data and %sregister%s to make sure your users receive your articles via the followthis journal . Without registration the followthis topic box is working, but your articles are not delivered.', 'followthis' ), '<a href="http://www.followthis.de/publisher_edit/' . $this->get_setting( 'key' ) . '" onclick="javascript:followthis_set_ignore(\'not_active_warning\', \'not_active_warning\', \'' . wp_create_nonce( 'followthis-ignore' ) . '\');" target="_blank">', '</a>' ) . '</a>'
			. ' <a href="javascript:followthis_set_ignore(\'not_active_warning\', \'not_active_warning\', \'' . wp_create_nonce( 'followthis-ignore' ) . '\');" class="button">' . __( 'I know, close', 'followthis' ) . '</a>'
			. '</p></div>';
	}

	/**
	 * Register publisher
	 * @since 1.0
	 */
	function registerPublisher() {
		// get current blogname for followthis registration
		$blogname = get_option( 'blogname' ) ? get_option( 'blogname' ) : false;

		// if blogname is not set try to get the blog description instead
		if ( ! $blogname )
			$blogname = get_option( 'blogdescription' ) ? get_option( 'blogdescription' ) : '';

		// get current site url
		$prefix_url = get_option( 'siteurl' );

		// register publisher on followthis backend
		$response = $this->api->registerPublisher( $blogname, $prefix_url );

		if ( $response ) {
			return $response;
		} else {
			return false;
		}

	}

	/**
	 * Register publisher
	 * @since 1.0
	 */
	function publishVerification( $key, $url ) {

		$response = $this->api->publisherVerification( $key, $url );

		return $response ? $response : false;
	}

	/**
	 * write verification file
	 * @since 1.0
	 */
	function createVerificationFile( $name, $content ) {
		$file   = dirname( __FILE__ ) . '/' . $name . '.html';
		$handle = fopen( $file, 'w' );
		$result = fwrite( $handle, $content );
		fclose( $handle );

		return $result;
	}

	/**
	 * @param $string
	 *
	 * @since 1.0
	 *
	 * @return array array with tags
	 */
	function extractCommonWords( $string ) {
		$stopWords = array( 'nichts', 'gleich' );

		$string = preg_replace( '/\s\s+/i', '', $string ); // replace whitespace
		$string = trim( $string ); // trim the string
		$string = preg_replace( '/[^a-zA-Z0-9 -]/', '', $string ); // only take alphanumerical characters, but keep the spaces and dashes too…
		$string = strtolower( $string ); // make it lowercase

		preg_match_all( '/\b.*?\b/i', $string, $matchWords );
		$matchWords = $matchWords[0];

		foreach ( $matchWords as $key => $item ) {
			if ( $item == '' || in_array( strtolower( $item ), $stopWords ) || strlen( $item ) <= 4 || strlen( $item ) >= 26 ) {
				unset( $matchWords[$key] );
			}
		}
		$wordCountArr = array();
		if ( is_array( $matchWords ) ) {
			foreach ( $matchWords as $key => $val ) {
				$val = strtolower( $val );
				if ( isset( $wordCountArr[$val] ) ) {
					$wordCountArr[$val] ++;
				} else {
					$wordCountArr[$val] = 1;
				}
			}
		}
		arsort( $wordCountArr );
		$wordCountArr = array_slice( $wordCountArr, 0, 15 );

		return $wordCountArr;
	}

	/**
	 * @since 1.0
	 */
	public function add_tagging_meta_box() {
		add_meta_box(
			'followthis_tagging_meta_box_name'
			, __( 'followthis tags', 'followthis' )
			, array( &$this, 'render_tagging_meta_box_content' )
			, 'post'
			, 'side'
			, 'default'
		);
	}

	/**
	 * @since 1.0
	 */
	public function render_tagging_meta_box_content() {
		global $post;

		$added_tags             = wp_get_post_tags( $post->ID );
		$followthis_deactivated = get_post_meta( $post->ID, '_followthis_deactivate', true );

		$class = $followthis_deactivated ? 'followthis-hide' : 'followthis-show';

		echo '<div class="followthis-suggestions ' . $class . '">';
		if ( count( $added_tags ) == 0 ) {
			echo '<p class="followthis_warning">' . __( 'Choose tags for your readers to follow. Otherwise tags are taken from the post automatically.', 'followthis' ) . '</p>';
		}

		$string = $post->post_title . ' ' . $post->post_content;
		$taggs  = $this->extractCommonWords( $string );

		$already_added = array();
		foreach ( $added_tags as $added_tag ) {
			array_push( $already_added, $added_tag->name );
		}

		echo '<p>';
		_e( 'Click tag to add.', 'followthis' );
		echo '</p>';
		echo '<p>';
		$counter = 0;
		foreach ( $taggs as $tag => $count ) {
			if ( ! in_array( $tag, $already_added ) ) {
				echo '<a href="#" class="addAsSuggestedFollowThisTag">' . $tag . '</a> ';
				$counter ++;
			}
		}
		echo '</p>';
		if ( $counter == 0 ) {
			echo '<p><i>' . __( 'No more suggested tags or you haven\'t save the post.', 'followthis' ) . '</i></p>';
		}
		echo '</div>';

		wp_nonce_field( plugin_basename( __FILE__ ), 'followthis_noncename' );
		echo '<p><input type="checkbox" name="followthis_deactivate" id="followthis_deactivate" value="1" ' . checked( 1, $followthis_deactivated, false ) . ' /> <label for="followthis_deactivate">' . __( 'Deactivate followthis topic box for this post', 'followthis' ) . '</label></p>';
	}

	/**
	 * @since 1.0
	 *
	 * @param $post_id
	 */
	function save_postdata( $post_id ) {

		// verify this came from the our screen and with proper authorization,
		// because save_post can be triggered at other times
		if ( ! wp_verify_nonce( $_POST['followthis_noncename'], plugin_basename( __FILE__ ) ) )
			return;

		// Check permissions
		if ( 'post' == $_POST['post_type'] ) {
			if ( ! current_user_can( 'edit_post', $post_id ) )
				return;
		}

		if ( isset( $_POST['followthis_deactivate'] ) ) {
			update_post_meta( $post_id, '_followthis_deactivate', 1 );
		} else {
			update_post_meta( $post_id, '_followthis_deactivate', 0 );
		}

	}

	/**
	 * Generate followthis widget
	 * @since 1.0
	 */
	function followthis_add_post_content( $content ) {
		global $post;
		$ft_activated = get_post_meta( $post->ID, '_followthis_deactivate', true );
		if ( is_single() && ! $ft_activated ) {
			$content .= '<div id="ft-widget"></div>';
		}
		return $content;
	} // end filter_method_name

	/**
	 * @since 1.0
	 */
	function add_ft_keywords() {
		global $post;
		$tags       = wp_get_post_tags( $post->ID );
		$final_tags = '';
		if ( count( $tags ) > 0 ) {
			foreach ( $tags as $tag ) {
				$final_tags .= $tag->name . ', ';
			}
		} else {
			$content = get_the_title( $post->ID ) . ' ' .  $post->post_content;
			$tags    = $this->extractCommonWords( $content );
			if ( is_array( $tags ) ) {
				foreach ( $tags as $key => $val ) {
					$final_tags .= $key . ', ';
				}
			}

			$final_tags .= $this->get_setting( 'keywords' );
		}

		echo '<meta name="ft_keywords" content="' . $final_tags . '"/>';
	}

	/**
	 * loop through all posts and update posts with not tags
	 * @since 1.0
	 */
	static function retag_posts( $default_keywords = '' ) {
		$args = array(
			'post_type'      => 'post',
			'posts_per_page' => - 1,
			'post_status'    => 'publish'
		);
		// get all posts
		$all_posts = new WP_Query( $args );

		// loop trough all posts
		while ( $all_posts->have_posts() ) : $all_posts->the_post();
			// loop only posts with zero tags
			if ( count( wp_get_post_tags( get_the_ID() ) ) < 1 ) {
				$content = get_the_title() . ' ' . get_the_content(); // concatenated post title and post content

				$all_keywords = array();
				$keywords     = self::extractCommonWords( $content ); // extract keywords from post content
				foreach ( $keywords as $key => $val ) {
					array_push( $all_keywords, $key );
				}

				$new_keywords = explode( ',', $default_keywords ); // get default keywords from option
				$all_keywords     = array_map( 'trim', array_merge( $all_keywords, $new_keywords ) ); // merge extracted keywords and default keywords and trim spaces

				wp_set_object_terms( get_the_ID(), $all_keywords, 'post_tag', false ); // set terms to post
			}
		endwhile;
	}
} // end class

$plugin_name = new Followthis();
